require 'active_support/concern'

module Stackd
  module Concerns
    module TattrAccessor
      extend ActiveSupport::Concern

      included do
        @_tattrs = {}
      end

      class_methods do
        def tattr_accessor tattrs
          tattrs.each do |tattr, type|
            attr_reader tattr

            @_tattrs[tattr] = type

            define_method :"#{tattr}=" do |value|
              unless value.nil? || self.class.tattr_type_match?(tattr, value)
                tattr_type = self.class.tattr_type tattr
                raise TypeMismatchError.new [
                  "tattr: #{tattr}",
                  "tattr_type: #{tattr_type}",
                  "value: #{value.inspect}"
                ].join ', '
              end

              instance_variable_set :"@#{tattr}", value
            end
          end
        end

        def tattr_type_match? tattr, value
          value.is_a? tattr_type(tattr)
        end

        def tattr_type tattr
          @_tattrs[tattr]
        end
      end

      def tattr? key
        self.class.instance_variable_get(:@_tattrs).has_key? key
      end

      class TypeMismatchError < StandardError
      end
    end
  end
end
