require 'active_support/core_ext/object/to_query'

module Stackd
  class Token
    include Concerns::TattrAccessor
    include Concerns::RequireAttr

    @_on_grant_callbacks = []

    tattr_accessor client: Client,
                   access_token: String,
                   token_type: String,
                   expires_in: Integer,
                   refresh_token: String,
                   scope: String,
                   service_id: String,
                   user_id: String

    def self.on_grant &block
      @_on_grant_callbacks.push block
    end

    def initialize client, attrs = {}
      self.client = client

      attrs.each do |key, val|
        public_send :"#{key}=", val if tattr? key
      end

      begin
        require_attr! :client, :access_token, :token_type
      rescue Concerns::RequireAttr::AttrNotSetError => e
        raise ArgumentError.new e.message
      end
    end

    def get *args
      Util::HTTP.get self, get_url(*args)
    end

    def post *args
      url_args = reject_body_arg *args
      body_arg = get_body_arg *args
      content_type = content_type_for body_arg

      Util::HTTP.post self, get_url(*url_args), body_arg, content_type
    end

    def put *args
      url_args = reject_body_arg *args
      body_arg = get_body_arg *args
      content_type = content_type_for body_arg

      Util::HTTP.put self, get_url(*url_args), body_arg, content_type
    end

    def patch *args
      url_args = reject_body_arg *args
      body_arg = get_body_arg *args
      content_type = content_type_for body_arg

      Util::HTTP.patch self, get_url(*url_args), body_arg, content_type
    end

    def delete *args
      url_args = reject_body_arg *args
      body_arg = get_body_arg *args
      content_type = content_type_for body_arg

      Util::HTTP.delete self, get_url(*url_args), body_arg, content_type
    end

    private

    def get_url *args
      args.inject Stackd.config.api_url do |url, part|
        if part.is_a? Hash
          "#{url}?#{part.to_query}"
        else part = part.to_s
          if part.index(Stackd.config.api_url) == 0
            part
          elsif part.index('/') == 0
            "#{url}#{part}"
          else
            "#{url}/#{part}"
          end
        end
      end
    end

    def reject_body_arg *args
      args.reject {|a| a.is_a? Hash }
    end

    def get_body_arg *args
      args.last if args.last.is_a? Hash
    end

    def content_type_for body_arg
      :json unless body_arg.nil?
    end
  end
end
